#
# Standard GNU Make non-recursive prologue. Pushes callers $(d) onto a stack,
# for re-entrant Rules.mk processing.
#
sp		:= $(sp).x
dirstack_$(sp)	:= $(d)
d		:= $(dir)


# Our parent directory
$(d)/..		:= $(subst /$(notdir $(d)),,$(d))


#
# Make sure we have our defaults.
#
include $($(d)/..)/mk/Variables.mk


OBJS_$(d)	:= $(d)/arena.o $(d)/pool.o $(d)/proto.o $(d)/util.o
SRCS_$(d)	:= $(d)/arena.c $(d)/pool.c $(d)/proto.c $(d)/util.c
INCS_$(d)	:= $(d)/arena.h $(d)/pool.h $(d)/proto.h $(d)/rbits.h $(d)/util


CFLAGS_$(d)	:= $(CFLAGS)
CPPFLAGS_$(d)	:= -I$($(d)/..)/compat $(CPPFLAGS)

M4FLAGS_$(d)	:= -I$(d) $(M4FLAGS)


#
# Compilation Targets
#
$(d)/arena.h $(d)/pool.h $(d)/proto.h $(d)/align.h: $(d)/%.h: $(d)/%.h.in $($(d)/..)/m4/macros.m4
	$(M4) $(M4FLAGS_$(@D)) -Dscope=private $(@D)/../m4/macros.m4 $< > $@

$(d)/arena.o: $(d)/arena.c $(d)/arena.h $(d)/proto.h $(d)/rbits.h $(d)/util.h $(d)/align.h
	$(CC) -c $(CPPFLAGS_$(@D)) $(CFLAGS_$(@D)) -o $@ $<

$(d)/pool.o: $(d)/pool.c $(d)/pool.h $(d)/proto.h $(d)/rbits.h $(d)/util.h $(d)/align.h
	$(CC) -c $(CPPFLAGS_$(@D)) $(CFLAGS_$(@D)) -o $@ $<

$(d)/proto.o: $(d)/proto.c $(d)/proto.h $(d)/rbits.h $(d)/align.h
	$(CC) -c $(CPPFLAGS_$(@D)) $(CFLAGS_$(@D)) -o $@ $<

$(d)/util.o: $(d)/util.c $(d)/util.h $(d)/proto.h
	$(CC) -c $(CPPFLAGS_$(@D)) $(CFLAGS_$(@D)) -o $@ $<

$(d)/libarena.a: $(OBJS_$(d))
	$(AR) $(ARFLAGS) $@ $^
	$(RANLIB) $(RANLIBFLAGS) $@

all: $(d)/libarena.a


#
# Installation Targets
#
$(DESTDIR)$(includedir)/arena: | $(DESTDIR)$(includedir)
	$(MKDIR) $@

$(DESTDIR)$(includedir)/arena/proto.h $(DESTDIR)$(includedir)/arena/pool.h: $(DESTDIR)$(includedir)/arena/%.h: $(d)/%.h.in $($(d)/..)/m4/macros.m4 | $(DESTDIR)$(includedir)/arena
	$(M4) $(M4FLAGS_$(<D)) -Dscope=public $(<D)/../m4/macros.m4 $< > $@

$(DESTDIR)$(includedir)/arena/util.h $(DESTDIR)$(includedir)/arena/rbits.h: $(DESTDIR)$(includedir)/arena/%.h: $(d)/%.h | $(DESTDIR)$(includedir)/arena
	$(INSTALL_DATA) $< $@

$(DESTDIR)$(libdir)/libarena.a: $(d)/libarena.a | $(DESTDIR)$(libdir)
	$(INSTALL_DATA) $< $@

-larena: $(DESTDIR)$(libdir)/libarena.a $(DESTDIR)$(includedir)/arena/proto.h \
         $(DESTDIR)$(includedir)/arena/pool.h $(DESTDIR)$(includedir)/arena/util.h \
         $(DESTDIR)$(includedir)/arena/rbits.h

install: -larena


#
# Uninstall targets
#
.PHONY: $(d)/uninstall

$(d)/uninstall:
	rm -fr $(DESTDIR)$(includedir)/arena
	rm -f $(DESTDIR)$(libdir)/libarena.a

uninstall: $(d)/uninstall


#
# Clean targets
#
.PHONY: $(d)/clean

$(d)/clean:
	rm -f $(@D)/arena $(@D)/pool $(@D)/pool.h \
		$(@D)/arena.h $(@D)/proto.h $(@D)/align.h $(@D)/*.o $(@D)/*.a

clean: $(d)/clean


#
# Standard GNU Make non-recursive epilogue. Restores callers $(d), for
# re-entrant Rules.mk processing.
#
d	:= $(dirstack_$(sp))
sp	:= $(basename $(sp))

